#include <condition_variable>
#include <chrono>
#include <iostream>
#include <mutex>
#include <thread>
#include <vector>

struct Barrier {
	explicit Barrier(int w) : _workers(w), _arrived(0) {}
	void wait() {
		std::unique_lock< std::mutex > lk(_mtx);
		if (++_arrived == _workers) {
			std::cout << "--- barrier ---\n";
			lk.unlock();
			_cv.notify_all();
		} else {
			_cv.wait(lk, [this]{ return _arrived == _workers; });
			// this is the same as:
			// while (_arrived != _workers) {
			//	_cv.wait(lk);
			// }
		}
	}
private:
	const int _workers;
	int _arrived;
	std::condition_variable _cv;
	std::mutex _mtx;
};

void worker(Barrier& barrier, int id, int ms) {
	std::cout << "Worker " << id << " started.\n";
	std::this_thread::sleep_for(std::chrono::milliseconds( ms ));
	std::cout << "Worker " << id << " has reached the barrier.\n";
	barrier.wait();
	std::cout << "Worker " << id << " after the barrier.\n";
}

int main() {
	const int num_workers = 10;
	Barrier barrier( num_workers );
	std::vector< std::thread > threads;
	for (int i = 0; i < num_workers; ++i) {
		threads.emplace_back( worker, std::ref(barrier), i,
		                      i * 100 );
	}

	for (auto& thr : threads) {
		thr.join();
	}
}
