#include <iostream>
#include <mutex>
#include <thread>
#include <vector>

std::mutex cout_mutex;

struct Item {
	Item(int id) : id(id) {}
	void doWork() {
		std::this_thread::sleep_for(std::chrono::milliseconds(200));
		std::lock_guard< std::mutex > lock( cout_mutex );
		std::cout << id << " done\n";
	}
private:
	int id;
};

std::mutex mutex;
std::vector< Item > work;
size_t next = 0;

Item* getNextItem() {
	std::lock_guard< std::mutex > lock( mutex );
	return work.size() <= next ? nullptr : &work[next++];
}

void worker() {
	while ( Item* item = getNextItem() ) {
		item->doWork();
	}
}

int main() {
	for (int i = 0; i < 20; ++i) {
		work.emplace_back( i );
	}
	std::vector< std::thread > threads;
	for (int i = 0; i < 4; ++i) {
		threads.emplace_back( worker );
	}
	std::cout << "main\n";
	for (auto& thr : threads) {
		thr.join();
	}
}
