#include <boost/asio/io_service.hpp>
#include <boost/asio/ip/tcp.hpp>
#include <boost/asio/write.hpp>

#include <array>
#include <iostream>
#include <string>
#include <thread>

using namespace boost::asio::ip;

class WebClient {
	boost::asio::io_service io_service;
	boost::asio::io_service::work work{ io_service };
	tcp::socket tcp_socket{ io_service };
	tcp::resolver resolver{ io_service };
	std::thread thread;
	std::array<char, 4096> buffer;

public:
	WebClient() : thread{ [this]{ io_service.run(); } } {}

	void read_handler(const boost::system::error_code& ec,
	                  std::size_t len) {
		if (ec) {
			// this is possibly eof or tcp_socket closing
			return;
		}
		std::cout.write(buffer.data(), len);
		tcp_socket.async_read_some(boost::asio::buffer(buffer),
			[this](auto&&... args){ read_handler(args...); });
	}

	void connect_handler(const boost::system::error_code& ec) {
		if (ec) {
			std::cerr << "Could not connect: "
			          << ec.message() << '\n';
			return;
		}
		static std::string cmd = "GET / HTTP/1.0\r\n\r\n";
		boost::asio::write(tcp_socket, boost::asio::buffer(cmd));
		tcp_socket.async_read_some(boost::asio::buffer(buffer),
			[this](auto&&... args){ read_handler(args...); });
	}

	void resolve_handler(const boost::system::error_code& ec,
	                     tcp::resolver::iterator it) {
		if (ec) {
			std::cerr << "Could not resolve URL: "
			          << ec.message() << '\n';
			return;
		}
		tcp_socket.close();
		tcp_socket.async_connect(*it,
			[this](auto&&...args){ connect_handler(args...); });
	}

	void get(const std::string& url) {
		resolver.async_resolve(url, "http",
			[this](auto&&... args){ resolve_handler(args...); });
	}

	~WebClient() {
		io_service.stop();
		thread.join();
	}
};


int main() {
	WebClient client;
	std::string url;
	while (std::getline(std::cin, url)) {
		client.get(url);
	}
}
